#include "dma_demo.h"
#include "spi_demo.h"
unsigned int CTRL_DATA_SPI10 = 5;
unsigned int CTRL_DATA_SPI11 = 6;


/***********************************************************************************************************************
* Function Name: DMA_Sspi_Write
* @brief  config DMA for spi write
* @param  src_adr: dma source address
* @param  dst_adr: dma destine address
* @param  count: dma transform num
* @return None
***********************************************************************************************************************/
void DMA_Sspi_Write(DMA_VECTOR_t dma_vector, uint8_t ctrl_data, DMA_Mode_t mode, void *src_adr, void *dst_adr, uint16_t count)
{
    DMA_InitTypeDef  DMA_InitStructure = {0};

    DMA_InitStructure.DMA_Vector = dma_vector;    //ݹѡͬdma
    DMA_InitStructure.DMA_CtrlId = ctrl_data;     //ѡ
    DMA_InitStructure.DMA_SrcAddr = (uint32_t)src_adr;  //dmaԴַ
    DMA_InitStructure.DMA_DstAddr = (uint32_t)dst_adr;  //dmaĿַ
    DMA_InitStructure.DMA_BufferSize = count;
    DMA_InitStructure.DMA_SrcInc = DMA_SrcInc_Enable;//Դַģʽ
    DMA_InitStructure.DMA_DstInc = DMA_DstInc_Disable;//Ŀ̶ַ
    DMA_InitStructure.DMA_DataSize = DMA_DataSize_Byte;//ݳѡ
    DMA_InitStructure.DMA_Mode = mode;//ͨģʽ

	DMA_Init(&DMA_InitStructure);

	DMA_Start(DMA_InitStructure.DMA_Vector);

}


/***********************************************************************************************************************
* Function Name: DMA_Sspi_Read
* @brief  config DMA for spi read
* @param  src_adr: dma source address
* @param  dst_adr: dma destine address
* @param  count: dma transform num
* @return None
***********************************************************************************************************************/
void DMA_Sspi_Read(DMA_VECTOR_t dma_vector, uint8_t ctrl_data, DMA_Mode_t mode, void *src_adr, void *dst_adr, uint16_t count)
{
    DMA_InitTypeDef  DMA_InitStructure = {0};
    static uint8_t dummy_sio11 = 0xFFU;
    /* read receive data with DMA */
    DMA_InitStructure.DMA_Vector = dma_vector;
    DMA_InitStructure.DMA_CtrlId = ctrl_data;
    DMA_InitStructure.DMA_SrcAddr = (uint32_t)src_adr;  //dmaԴַ
    DMA_InitStructure.DMA_DstAddr = (uint32_t)dst_adr;  //dmaĿַ
    DMA_InitStructure.DMA_BufferSize = count;
    DMA_InitStructure.DMA_SrcInc = DMA_SrcInc_Disable;//Դ̶ַģʽ
    DMA_InitStructure.DMA_DstInc = DMA_DstInc_Enable;//Ŀַģʽ
    DMA_InitStructure.DMA_DataSize = DMA_DataSize_Byte;//ݳѡ
    DMA_InitStructure.DMA_Mode = mode; //ͨģʽ
    DMA_InitStructure.DMA_ChainTrans = DMA_ChainTrans_Head; //

    DMA_Init(&DMA_InitStructure);
    /* write dummy transfer data with DMA */
    DMA_InitStructure.DMA_CtrlId = ctrl_data + 1;
    DMA_InitStructure.DMA_SrcAddr = (uint32_t)&dummy_sio11;  //dmaԴַ
    DMA_InitStructure.DMA_DstAddr = (uint32_t)src_adr;  //dmaĿַ
    DMA_InitStructure.DMA_BufferSize = count;
    DMA_InitStructure.DMA_SrcInc = DMA_SrcInc_Disable;//Դ̶ַģʽ
    DMA_InitStructure.DMA_DstInc = DMA_DstInc_Disable;//Ŀ̶ַģʽ
    DMA_InitStructure.DMA_DataSize = DMA_DataSize_Byte;//ݳѡ
    DMA_InitStructure.DMA_Mode = mode; //ͨģʽ
    DMA_InitStructure.DMA_ChainTrans = DMA_ChainTrans_Tail; //ر
    DMA_Init(&DMA_InitStructure);

	DMA_Start(DMA_InitStructure.DMA_Vector);
}

/***********************************************************************************************************************
* Function Name: DMA_Sspi_WriteRead
* @brief  DMA peripheral write data and recive data at the same time
* @brief  config DMA for spi read
* @param  src_adr: dma source address
* @param  dst_adr: dma destine address
* @param  count: dma transform num
* @return None
***********************************************************************************************************************/
void DMA_Sspi_WriteRead(DMA_VECTOR_t dma_vector, uint8_t ctrl_data, DMA_Mode_t mode, void *src_adr, void *dst_adr, uint16_t count)
{
    DMA_InitTypeDef  DMA_InitStructure = {0};

    /* read receive data with DMA */
    DMA_InitStructure.DMA_Vector = dma_vector;
    DMA_InitStructure.DMA_CtrlId = ctrl_data;
    DMA_InitStructure.DMA_SrcAddr = (uint32_t)&SSPI11_SDR;  //dmaԴַ
    DMA_InitStructure.DMA_DstAddr = (uint32_t)dst_adr;  //dmaĿַ
    DMA_InitStructure.DMA_BufferSize = count;
    DMA_InitStructure.DMA_SrcInc = DMA_SrcInc_Disable;//Դ̶ַģʽ
    DMA_InitStructure.DMA_DstInc = DMA_DstInc_Enable;//Ŀַģʽ
    DMA_InitStructure.DMA_DataSize = DMA_DataSize_Byte;//ݳѡ
    DMA_InitStructure.DMA_Mode = mode; //ͨģʽ
    DMA_InitStructure.DMA_ChainTrans = DMA_ChainTrans_Head; //ͷڵ
	
    DMA_Init(&DMA_InitStructure); 
    /* write dummy transfer data with DMA */
    DMA_InitStructure.DMA_CtrlId = ctrl_data + 1;
    DMA_InitStructure.DMA_SrcAddr = (uint32_t)src_adr;  //dmaԴַ
    DMA_InitStructure.DMA_DstAddr = (uint32_t)&SSPI11_SDR;  //dmaĿַ
    DMA_InitStructure.DMA_BufferSize = count;
    DMA_InitStructure.DMA_SrcInc = DMA_SrcInc_Enable;//Դ̶ַģʽ
    DMA_InitStructure.DMA_DstInc = DMA_DstInc_Disable;//Ŀַģʽ
    DMA_InitStructure.DMA_DataSize = DMA_DataSize_Byte;//ݳѡ
    DMA_InitStructure.DMA_Mode = mode; //ͨģʽ
    DMA_InitStructure.DMA_ChainTrans = DMA_ChainTrans_Tail; //βڵ
	
	DMA_Init(&DMA_InitStructure);
	DMA_Start(DMA_InitStructure.DMA_Vector);
}
