/*******************************************************************************
*                  COPYRIGHT (C) 2021 CMS Technologies Ltd.                    *
*                                                                              *
********************************************************************************
* FileName      : spihs_demo.c                                                 *
* Author        :                                                              *
* Version       : 1.0                                                          *
* Date          : 2021.08.13                                                   *
* Description   :                                                              *
* Function List :                                                              *
********************************************************************************/
#include "spihs_demo.h"
#include "dma_demo.h"
volatile uint8_t *gp_spi_rx_address;       /* spi receive buffer address */
volatile uint16_t g_spi_rx_count;          /* spi receive data count */
uint8_t g_spi_tx_end = 0, g_spi_rx_end = 0;

void NSS0_set()
{
    GPIO_SetBits(GPIO_PORT5, GPIO_Pin_4);
}
void NSS0_clr()
{
    GPIO_ResetBits(GPIO_PORT5, GPIO_Pin_4);
}
void NSS1_set()
{
    GPIO_SetBits(GPIO_PORT6, GPIO_Pin_3);
}
void NSS1_clr()
{
    GPIO_ResetBits(GPIO_PORT6, GPIO_Pin_3);
}
/***********************************************************************************************************************
* Function Name: Spihs_Init
* @brief high speed spi init
* @param
* @return
***********************************************************************************************************************/
void Spihs_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    SPI_InitTypeDef  SPI_InitStructure = {0};

    /*NSS GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_4;   //NSSѡģʽʼʱø/͵ƽΪ
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level = GPIO_Level_HIGH;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT5, &GPIO_InitStruct);

    /*SCLK GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_5;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_Init(GPIO_PORT5, &GPIO_InitStruct);

    /*MOSI GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_7;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_InitStruct.GPIO_Level = GPIO_Level_LOW;
    GPIO_Init(GPIO_PORT5, &GPIO_InitStruct);

    /*MISO GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_6;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_IN;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT5, &GPIO_InitStruct);

    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;           //SPIģʽ:ΪSPI SPI_Mode_Master
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;       //SPIݴС:SPIͽ8λ֡ṹ
    SPI_InitStructure.SPI_CPOL      = SPI_CPOL_Low;         //ʱڵ͵ƽʱΪ̬ ģʽ0
    SPI_InitStructure.SPI_CPHA      = SPI_CPHA_1Edge;       //ڵһز   ģʽ0
    SPI_InitStructure.SPI_NSS       = SPI_NSS_Hard ;        //ʹNSS
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32; //ѡspiʱ fmclk =clk/2^3
    SPI_InitStructure.SPI_FirstBit  = SPI_FirstBit_MSB;          //ѡbitȴ
    SPI_Init(SPIHS0,&SPI_InitStructure);
    SPI_BiDirectionalLineConfig(SPIHS0,SPI_Direction_TxRx);

    ISR_Register(SPI0_IRQn, spi0hs_interrupt);   //жϷ·ע

    NSS.Active = NSS0_clr;                             //NSSƬѡʼ
    NSS.Inactive = NSS0_set;
    NSS.Inactive();                                  //NSSʼ״̬
}


/***********************************************************************************************************************
* Function Name: Spihs_Write
* @brief  spi write data
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Write(uint8_t data)
{
    SPI_TransmitByte(SPIHS0,data);
}

/***********************************************************************************************************************
* Function Name: Spihs_Read
* @brief  spi read data
* @param  None
* @return None
***********************************************************************************************************************/
uint8_t Spihs_Read(void)
{
    return SPI_ReceiveByte(SPIHS0);
}

/***********************************************************************************************************************
* Function Name: SPI_Start
* @brief  high speed spi interrupt start
* @param  SPI_FUNC_t
* @return
***********************************************************************************************************************/
void SPI_Start()
{
    INTC_ClearPendingIRQ(SPI0_IRQn);
    NVIC_ClearPendingIRQ(SPI0_IRQn);
    INTC_EnableIRQ(SPI0_IRQn);
    NVIC_EnableIRQ(SPI0_IRQn);
    SPI_Cmd(SPIHS0,ENABLE);        //spi
}

/***********************************************************************************************************************
* Function Name: SPI_Stop
* @brief  high speed spi interrupt stop
* @param  SPI_FUNC_t
* @return
***********************************************************************************************************************/
void SPI_Stop()
{
    SPI_Cmd(SPIHS0,DISABLE);        //رspi
    INTC_DisableIRQ(SPI0_IRQn);
    NVIC_DisableIRQ(SPI0_IRQn);
    INTC_ClearPendingIRQ(SPI0_IRQn);
    NVIC_ClearPendingIRQ(SPI0_IRQn);

}

/***********************************************************************************************************************
* Function Name: Spihs_Int_Write
* @brief  high speed spi write data by interrupt
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Int_Write(uint8_t *tx_buf, uint16_t tx_num)
{
    SPI_Start();

    g_spi_tx_end = 0;
    pData.data = &tx_buf[1];    //send buffer pointer
    pData.len  = tx_num - 1; //set send data count
    pData.flag = INT_TX;   //set data direction

    SPI_SendData(SPIHS0,tx_buf[0]);
}

/***********************************************************************************************************************
* Function Name: Spihs_Int_TxRx
* @brief  high speed spi write data and read data
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Int_TxRx(uint8_t *tx_buf, uint16_t tx_num, uint8_t *rx_buf, uint16_t rx_num)
{
    SPI_Start();
    SPI_BiDirectionalLineConfig(SPIHS0,SPI_Direction_TxRx);
    g_spi_tx_end = 0;
    pData.data = &tx_buf[1];    //send buffer pointer
    pData.len  = tx_num;  //set send data count
    pData.flag = INT_TX;   //set data direction

    g_spi_rx_count = rx_num;    /* receive data count */
    gp_spi_rx_address = rx_buf; /* receive buffer pointer */
    SPI_SendData(SPIHS0,tx_buf[0]);
}
/***********************************************************************************************************************
* Function Name: Spihs_Int_Read
* @brief  high speed spi read data by interrupt
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Int_Read(uint8_t *rx_buf, uint16_t rx_num)
{
    uint32_t dummy_sdr;

    SPI_Start();

    SPI_BiDirectionalLineConfig(SPIHS0,SPI_Direction_Rx);

    g_spi_rx_end = 0;
    pData.data = rx_buf;   //read buffer pointer
    pData.len  = rx_num;   //set receive data count
    pData.flag = INT_RX;   //set data direction

    dummy_sdr = SPI_ReceiveData(SPIHS0);
}

/***********************************************************************************************************************
* Function Name: Spi_Dma_Write
* @brief  spi write data by dma
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Dma_Write(uint8_t *tx_buf, uint16_t tx_num)
{
    uint8_t CTRL_DATA_SPI = 12;

    SPI_BiDirectionalLineConfig(SPIHS0,SPI_Direction_TxRx);
    Dma_Config(DMA_VECTOR_SPIHS0, CTRL_DATA_SPI, DMA_Mode_Normal, tx_buf + 1, (void *)&SPI0_WRDR, tx_num-1); //config dma transmission

    g_spi_tx_end = 0;
    SPI_Start();

    SPI_SendData(SPIHS0,tx_buf[0]);
    pData.flag = INT_DMA;   //set data direction
}

/***********************************************************************************************************************
* Function Name: Spihs_Dma_Read
* @brief
* @param   rx_buf: read spi buffer
*          rx_num: the num of rx_buf
* @return
***********************************************************************************************************************/
void Spihs_Dma_Read(uint8_t *rx_buf, uint16_t rx_num)
{
    uint32_t dummy_sdr;
    uint8_t CTRL_DATA_SPI = 12;

    SPI_BiDirectionalLineConfig(SPIHS0,SPI_Direction_Rx);

    Dma_Config1(DMA_VECTOR_SPIHS0, CTRL_DATA_SPI, DMA_Mode_Normal, (void *)&SPI0_RDDR, rx_buf, rx_num); //config dma transmission

    SPI_Start();
    g_spi_rx_end = 0;
    dummy_sdr = SPI_ReceiveData(SPIHS0);
    pData.flag = INT_DMA;   //set data direction
    pData.len = 1;
}

/***********************************************************************************************************************
* Function Name: Spihs_Dma_TxRx
* @brief  SPI peripheral write data and recive data at the same time
* @param   tx_buf: tx_buf to be transmitted by the SPI peripheral
**           tx_num: num to be transmitted
**           rx_buf: received buffer by the SPI peripheral
**           rx_num: received num  by the SPI peripheral
* @return
***********************************************************************************************************************/
void Spihs_Dma_TxRx(uint8_t *tx_buf, uint16_t tx_num, uint8_t *rx_buf, uint16_t rx_num)
{
    uint8_t CTRL_DATA_SPI = 12;

    SPI_BiDirectionalLineConfig(SPIHS0,SPI_Direction_TxRx);
    DMA_Spi_WriteRead(DMA_VECTOR_SPIHS0, CTRL_DATA_SPI, DMA_Mode_Normal, tx_buf+ 1, rx_buf, tx_num);

    g_spi_tx_end = 0;
    SPI_Start();

    SPI_SendData(SPIHS0,tx_buf[0]);
    pData.flag = INT_DMA;   //set data direction
}

/***********************************************************************************************************************
* Function Name: spi0hs_interrupt
* @brief  high speed spi interrupt service routine
* @param  None
* @return None
***********************************************************************************************************************/
void spi0hs_interrupt(void *msg)
{
    volatile uint8_t sio_dummy;
    ATE_FRAME_t *pFrame = (ATE_FRAME_t *)msg;

    if (pFrame->flag == INT_TX)
    {
        if (pFrame->len > 0)
        {
            //          if (0U != pFrame->data)
            {
                SPI0_WRDR = *pFrame->data;
                pFrame->data++;
                pFrame->len--;
                *gp_spi_rx_address = SPI_ReceiveData(SPIHS0);
                gp_spi_rx_address++;

            }
            //          else
            //          {
            //              SPI_WRDR = 0xFFU;
            //          }
        }
        else
        {
            while (SPI0_BUSY);

            /* complete send */
            SPI_Stop();
            pFrame->len = 0;
            pFrame->flag = INT_IDLE;
            g_spi_tx_end = 1;
        }
    }

    if (pFrame->flag == INT_RX)
    {
        if (pFrame->len > 1)
        {
            *pFrame->data = SPI_ReceiveData(SPIHS0);
            pFrame->data++;
            pFrame->len--;
        }
        else if (pFrame->len == 1)
        {
            *pFrame->data =  SPI_ReceiveData(SPIHS0);
            pFrame->data++;
            pFrame->len--;
            {
                /* complete receive */
                SPI_Stop();
                pFrame->len = 0;
                pFrame->flag = INT_IDLE;
                g_spi_rx_end = 1;
            }
        }
    }

    if (pFrame->flag == INT_DMA)
    {
        if (pFrame->len == 1)
        {		
            SPI_Stop();
            g_spi_rx_end = 1;
        }

        while (SPI0_BUSY);

        SPI_Stop();
        g_spi_tx_end = 1;
    }

    INTC_ClearPendingIRQ(SPI0_IRQn);
}


