#include "drv_w25qxx.h"
#include "spi.h"
#include "spihs_demo.h"
#include "string.h"
#include "delay_demo.h"
//#define SPI_POLLING
#define SPI_WITH_DMA

uint16_t drv_w25qxx_read_device_id(void)
{
    uint16_t tmp = 0;
    static uint8_t rxbuf[6];
    static uint8_t txbuf[7] = {0x90, 0x00, 0x00, 0x00, 0xff, 0xff};
#ifdef  SPI_POLLING
    SPI_Cmd(ENABLE);

    SPI_TransmitByte(0x90);
    SPI_TransmitByte(0x00);
    SPI_TransmitByte(0x00);
    SPI_TransmitByte(0x00);
    tmp |= SPI_TransmitByte(0xFF) << 8;
    tmp |= SPI_TransmitByte(0xFF);

    SPI_Cmd(DISABLE);

    return tmp;
#else
#ifdef SPI_WITH_DMA
    Spihs_Dma_TxRx(txbuf, 6, rxbuf, 6);
    delayMS(4);
    while (g_spi_tx_end == 0);

    tmp |= (rxbuf[4] << 8);
    tmp |= rxbuf[5];
    return tmp;
#else
#if 1 /*ʹж϶ʽ*/
    SPI_Cmd(ENABLE);
    SPI_BiDirectionalLineConfig(SPI_Direction_TxRx);
    SPI_TransmitByte(0x90);
    SPI_TransmitByte(0x00);
    SPI_TransmitByte(0x00);
    SPI_TransmitByte(0x00);

    Spihs_Int_Read(rxbuf, 2);
    delayMS(1);
    while (g_spi_rx_end == 0);

    tmp |= (rxbuf[0] << 8);
    tmp |= rxbuf[1];

    return tmp;
#else/*ʹж϶дʽ*/
    Spihs_Int_TxRx(txbuf, sizeof(txbuf), rxbuf, sizeof(rxbuf));
    delayMS(1);
    while (g_spi_tx_end == 0);

    tmp |= (rxbuf[4] << 8);
    tmp |= rxbuf[5];
    return tmp;
#endif
#endif
#endif
}

uint32_t drv_w25qxx_read_jedec_id(void)
{
    uint32_t tmp = 0;
    static uint8_t rxbuf[5];
    static uint8_t txbuf[5] = {0x9F, 0xff, 0xff, 0xff, 0xff};
#ifdef  SPI_POLLING
    SPI_Cmd(ENABLE);

    SPI_TransmitByte(0x9F);
    tmp |= SPI_TransmitByte(0xFF) << 16;
    tmp |= SPI_TransmitByte(0xFF) << 8;
    tmp |= SPI_TransmitByte(0xFF);

    SPI_Cmd(DISABLE);

    return tmp;
#else
#ifdef SPI_WITH_DMA
    Spihs_Dma_TxRx(txbuf, 5, rxbuf, 5);
    delayMS(1);
    while (g_spi_tx_end == 0);

    tmp |= (rxbuf[1] << 16);
    tmp |= (rxbuf[2] << 8);
    tmp |= rxbuf[3];
    return tmp;
#else
#if 1 /*ʹж϶ʽ*/
    SPI_Cmd(ENABLE);
    SPI_BiDirectionalLineConfig(SPI_Direction_TxRx);
    SPI_TransmitByte(0x9F);

    Spihs_Int_Read(rxbuf, sizeof(rxbuf));

    while (g_spi_rx_end == 0);

    tmp |= (rxbuf[0] << 16);
    tmp |= (rxbuf[1] << 8);
    tmp |= rxbuf[2];
    return tmp;
#else/*ʹж϶дʽ*/
    Spihs_Int_TxRx(txbuf, sizeof(txbuf), rxbuf, sizeof(rxbuf));
    delayMS(1);
    while (g_spi_tx_end == 0);

    tmp |= (rxbuf[1] << 16);
    tmp |= (rxbuf[2] << 8);
    tmp |= rxbuf[3];
    return tmp;
#endif
#endif
#endif
}

void drv_w25qxx_read_uniq_id(uint8_t *buf)
{
    static uint8_t txbuf[13] = {0x4B, 0xff, 0xff, 0xff, 0xff};
    static uint8_t rxbuf[13];
#ifdef  SPI_POLLING
    SPI_Cmd(ENABLE);

    SPI_TransmitByte(0x4B);

    for (uint8_t i = 0; i < 4; i++)
    {
        SPI_TransmitByte(0x00);
    }

    for (uint8_t i = 0; i < 8; i++)
    {
        buf[i] = SPI_TransmitByte(0xFF);
    }

    SPI_Cmd(DISABLE);
#else
#ifdef SPI_WITH_DMA
    Spihs_Dma_TxRx(txbuf, 13, rxbuf, 13);
    delayMS(1);
    while (g_spi_tx_end == 0);

    memcpy(buf, &rxbuf[5], 8);
#else
#if 1 /*ʹж϶ʽ*/
    SPI_Cmd(ENABLE);
    SPI_BiDirectionalLineConfig(SPI_Direction_TxRx);
    SPI_TransmitByte(0x4B);

    for (uint8_t i = 0; i < 4; i++)
    {
        SPI_TransmitByte(0x00);
    }

    Spihs_Int_Read(buf, 8);
    delayMS(1);
    while (g_spi_rx_end == 0);

#else/*ʹж϶дʽ*/
    Spihs_Int_TxRx(txbuf, sizeof(txbuf), rxbuf, sizeof(rxbuf));
    delayMS(1);
    while (g_spi_tx_end == 0);

    memcpy(buf, &rxbuf[5], 8);
#endif
#endif
#endif
}
