/*******************************************************************************
*                  COPYRIGHT (C) 2021 CMS Technologies Ltd.                    *
*                                                                              *
********************************************************************************
* FileName      : spihs_demo.c                                                 *
* Author        :                                                              *
* Version       : 1.0                                                          *
* Date          : 2021.08.13                                                   *
* Description   :                                                              *
* Function List :                                                              *
********************************************************************************/
#include "spihs_demo.h"

void NSS1_set()
{
    GPIO_SetBits(GPIO_PORT5, GPIO_Pin_0);
}
void NSS1_clr()
{
    GPIO_ResetBits(GPIO_PORT5, GPIO_Pin_0);
}
/***********************************************************************************************************************
* Function Name: Spi0hs_Init
* @brief high speed spi init
* @param
* @return
***********************************************************************************************************************/
void Spihs_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    SPI_InitTypeDef  SPI_InitStructure = {0};
    ISR_InitTypeDef_t ISR_InitStructure;


    GPIO_SPI_GroupConfig(SPI_GROUP1);           //spiӳ飬鲻ͬӦGPIOͬ

    /*NSS GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_0;   //NSSѡģʽʼʱø/͵ƽΪ
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level = GPIO_Level_HIGH;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT5, &GPIO_InitStruct);

    /*SCLK GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_1;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_InitStruct.GPIO_Level = GPIO_Level_LOW;
    GPIO_Init(GPIO_PORT5, &GPIO_InitStruct);

    /*MOSI GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_6;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_InitStruct.GPIO_Level = GPIO_Level_LOW;
    GPIO_Init(GPIO_PORT1, &GPIO_InitStruct);

    /*MISO GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_7;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_IN;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT1, &GPIO_InitStruct);


    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;           //SPIģʽ:ΪSPI SPI_Mode_Master
    SPI_InitStructure.SPI_DataSize = SPIHS_Data_Bits_8;     //SPIݴС:SPIͽ8λ֡ṹ
    SPI_InitStructure.SPI_Phase_Mode = SPI_Phase_Mode0;     //ѡʹspiĹģʽ
    SPI_InitStructure.SPI_Prescaler = FCLK_Div_64;         //ѡspiʱ    fmclk =clk/2^4
    SPI_InitStructure.SPI_Bitorder = SPI_Bit_Msb;          //ѡbitȴ
    SPI_InitStructure.SPI_Int = SPI_Int_Enable;            //spiշж
    SPI_InitStructure.SPI_Int_Occur = SPI_Trans_End;       //ڴʱ,ж
    SPI_Init(SPI_HS, &SPI_InitStructure);

    ISR_InitStructure.IRQ_Channel = SPI_IRQn;
    ISR_InitStructure.IRQ[ISR_InitStructure.IRQ_Channel].intHandle = spi0hs_interrupt;//жϷ·ע
    ISR_Init(&ISR_InitStructure);


    NSS.Active = NSS1_clr;                             //NSSƬѡʼ
    NSS.Inactive = NSS1_set;
    NSS.Inactive();                                  //NSSʼ״̬
}


/***********************************************************************************************************************
* Function Name: Spi0hs_Write
* @brief  spi write data
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Write(uint8_t data)
{
    volatile uint8_t dummy;
    SPI->SDRO = data;

    while (SPI->SPIM & _0001_SPI_UNDER_EXECUTE);

    dummy = SPI->SDRI;
}

/***********************************************************************************************************************
* Function Name: Spi0hs_Read
* @brief  spi read data
* @param  None
* @return None
***********************************************************************************************************************/
uint8_t Spihs_Read(void)
{
    volatile uint8_t tmp;
    SPI->SDRO = 0xFF;

    while (SPI->SPIM & _0002_SPI_VALID_STORED);

    tmp = SPI->SDRI;

    return tmp;
}

/***********************************************************************************************************************
* Function Name: Spihs_Int_Write
* @brief  high speed spi write data by interrupt
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Int_Write(SPI_FUNC_t func, uint8_t *tx_buf, uint16_t tx_num)
{
    SPI_Start(func);

    if (func == SPI_HS)
    {
        SPI->SDRO = tx_buf[0];
    }

    tx_buf++ ;
    pData.data = tx_buf;    //send buffer pointer
    pData.len  = tx_num - 1; //set send data count
    pData.flag = INT_TX;   //set data direction
}
/***********************************************************************************************************************
* Function Name: Spihs_Int_Read
* @brief  high speed spi read data by interrupt
* @param  None
* @return None
***********************************************************************************************************************/
void Spihs_Int_Read(SPI_FUNC_t func, uint8_t *rx_buf, uint16_t rx_num)
{
    uint32_t dummy_sdr;

    if (func == SPI_HS)
    {
        SPI->SPIM &= ~(_0040_SPI_RECEPTION_TRANSMISSION | _0008_SPI_BUFFER_EMPTY);
    }

    SPI_Start(func);


    if (func == SPI_HS)
    {
        dummy_sdr = SPI->SDRI;
    }

    pData.data = rx_buf;   //read buffer pointer
    pData.len  = rx_num;   //set receive data count
    pData.flag = INT_RX;   //set data direction
}

/***********************************************************************************************************************
* Function Name: spi0hs_interrupt
* @brief  high speed spi interrupt service routine
* @param  None
* @return None
***********************************************************************************************************************/
void spi0hs_interrupt(void *msg)
{
    volatile uint8_t sio_dummy;
    ATE_FRAME_t *pFrame = (ATE_FRAME_t *)msg;

    if (pFrame->flag == INT_TX)
    {
        if (pFrame->len > 0)
        {
            if (0U != pFrame->data)
            {
                SPI->SDRO = *pFrame->data;
                pFrame->data++;
                pFrame->len--;
            }
            else
            {
                SPI->SDRO = 0xFFU;
            }
        }
        else
        {
            while (SPI->SPIM & _0001_SPI_UNDER_EXECUTE)
            {;}

            /* complete send */
            SPI_Stop(SPI_HS);
            pFrame->data = 0;
            pFrame->len = 0;
            pFrame->flag = INT_IDLE;
        }
    }

    if (pFrame->flag == INT_RX)
    {
        if (pFrame->len > 1)
        {
            if (0U != pFrame->data)
            {
                *pFrame->data = SPI->SDRI;
                pFrame->data++;
                pFrame->len--;
            }
            else
            {
                sio_dummy =  SPI->SDRI;
            }
        }
        else
        {
            if (pFrame->len == 1)
            {
                SPI->SPIM &= ~_0080_SPI_START_TRG_ON;

                if (0U != pFrame->data)
                {
                    *pFrame->data = SPI->SDRI;
                    pFrame->len--;
                }
                else
                {
                    sio_dummy = SPI->SDRI;
                }

            }

            if (0U != pFrame->data)
            {
                /* complete receive */
                SPI_Stop(SPI_HS);
                pFrame->data = 0;
                pFrame->len = 0;
                pFrame->flag = INT_IDLE;
            }
        }
    }

    INTC_ClearPendingIRQ(SPI_IRQn);
}


