/*******************************************************************************
*                  COPYRIGHT (C) 2021 CMS Technologies Ltd.                    *
*                                                                              *
********************************************************************************
* FileName      : tim_demo.c                                                  *
* Author        :                                                              *
* Version       : 1.0                                                          *
* Date          : 2021.08.13                                                   *
* Description   :                                                              *
* Function List :                                                              *
********************************************************************************/
#include "timm_demo.h"
#include "gpio_demo.h"
#include "uart_demo.h"

uint8_t interval_time_flag;
volatile uint16_t g_tmm0_intTaken_a = 0U;
/***********************************************************************************************************************
* Function Name: TMM0_IntervalTimer
* @brief  This function interval timer and generate interval interrupt.
* @note   Interval timer period calculation 1/clk *TMM_CHA_Pulse = timerValue (s)
* @return none
***********************************************************************************************************************/
void TMM0_IntervalTimer(uint8_t clk, uint16_t num)
{
	TMM_InitTypeDef TIMM_InitStructure={};

	TIMM_InitStructure.TMM_Select 	= TMM0;	
	TIMM_InitStructure.TMM_Channel  = TMM_Channel_A;
		
	TIMM_InitStructure.TMM_Clk = clk;   //specify the operation clk of tim
	TIMM_InitStructure.TMM_CHA_Pulse = num;
	TIMM_InitStructure.TMM_Mode = TMM_Mode_Compare;      // timer mode
	TIMM_InitStructure.TMM_CounterClear = TM_Clear_Match_TMGRA;  //when TM0 is matched with TMGRA, TM0 is cleared
	TIMM_InitStructure.TMM_Intp         = TMM_Int_IMIA; ///compare match interrupt enable A (IMIEA)

	TMM_Init(&TIMM_InitStructure);
	ISR_Register(TMM0_IRQn, tmm0_interrupt);  //жϷ·ע
}

/***********************************************************************************************************************
* Function Name: TMM0_Get_PulseWidth
* @brief  The TMM0 operates as pulse width measure.
* @param  clk - select count source clock
* 			@arg TMM_FCLK_Div1		
* 			@arg TMM_FCLK_Div2		
* 			@arg TMM_FCLK_Div4		
* 			@arg TMM_FCLK_Div8		
* 			@arg TMM_FCLK_Div32		
* 			@arg TMM_FCLK_TMCLK		
* @param  level - specify the measure level
* 			@arg TMM_Capture_Rising		
* 			@arg TMM_Capture_Falling		
* 			@arg TMM_Capture_Both		
* @return none
***********************************************************************************************************************/
void TMM0_Get_PulseWidth_Init(uint8_t clk, uint8_t capture_edge)
{
	uint32_t width;
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMM_InitTypeDef TIMM_InitStructure;


	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_7,GPIO_P17,GROUP_AF_ODEFAULT);  //P01 used as TMIOA0 input

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_7;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_IN;	
	GPIO_InitStruct.GPIO_Ctrl	= GPIO_Control_DIG;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;	
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);

	TIMM_InitStructure.TMM_Select 	= TMM0;	
	TIMM_InitStructure.TMM_Channel = TMM_Channel_A;
	TIMM_InitStructure.TMM_Clk = clk;   //specify the operation clk of tim
	TIMM_InitStructure.TMM_Mode    = TMM_Mode_Capture;      // Capture mode
	TIMM_InitStructure.TMM_IC.TMM_CHA.TMM_ICPolarity = TMM_Capture_Rising;//
	TIMM_InitStructure.TMM_IC.TMM_CHC_AS_BufferRegister = TMM_BufferRegister_Setting;//ͨͨCΪAĴϲ
	TIMM_InitStructure.TMM_CounterClear = TM_Clear_Disable;  // TM is not cleared
	TIMM_InitStructure.TMM_Intp     = TMM_Int_OVIE | TMM_Int_IMIA; ///input capture interrupt enable A (IMIEA)
	
	TMM_Init(&TIMM_InitStructure);
}


/***********************************************************************************************************************
* Function Name: TMM0_DoublePulseOutput
* @brief  The TMM operates as double pulse output.
* @param  clk - select count source clock
*		@arg TMM_FCLK_Div1
*		@arg TMM_FCLK_Div2		
*		@arg TMM_FCLK_Div4		
*		@arg TMM_FCLK_Div8		
*		@arg TMM_FCLK_Div32		
*		@arg TMM_FCLK_TMCLK		
* @param  m - the value of TMGRA register. it is the period of TMIOA0 and TMIOB0 output pulse.
* @param  n - the value of TMGRC register. it is the toggle point of TMIOA0.
* @param  p - the value of TMGRB register. it is the toggle point of TMIOB0.
* @param  q - the value of TMGRD register. it is the toggle point of TMIOB0.
* @return None
* @note   The value of m MUST be greater than the value of n, p, q. [m > m/p/q]
*            ^
*       FFFFH|
*            |
*TMGRA0 = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
*            |                    _| |                    _| |
*            |                  _|   |                  _|   |
*            |                _|     |                _|     |
*TMGRC0 = n-1|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _
*            |            _| |       |            _| |       |
*TMGRB0 = p-1|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _
*            |        _| |   |       |        _| |   |       |        
*            |      _|   |   |       |      _|   |   |       |      _  
*TMGRD0 = q-1|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|    
*            |  _| |     |   |       |  _| |     |   |       |  _|      
*       0000H|_|___|_____|___|_______|_|___|_____|___|_______|_|______
*                  |     |   |       |     |     |   |       |
*                  |     |   |_______|     |     |   |_______|
*     TMIOA0 ______|_____|___|       |_____|_____|___|       |________
*                  |     |   |       |__   |     |   |       |__
* TMSR0.IMFA ______|_____|___|_______|  |__|_____|___|_______|  |_____
*                  |     |   |__________   |     |   |__________
* TMSR0.IMFC ______|_____|___|          |__|_____|___|          |_____
*                  |     |                 |     |
*                  |_____|                 |_____|
*     TMIOB0 ______|     |_________________|     |____________________
*                  |     |______________   |     |______________
* TMSR0.IMFB ______|_____|              |__|_____|              |_____
*                  |____________________   |____________________
* TMSR0.IMFD ______|                    |__|                    |_____
*                                                 
***********************************************************************************************************************/
void TMM0_DoublePulseOutput(uint8_t clk, uint16_t period, uint16_t n, uint16_t p, uint16_t q )
{	
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMM_InitTypeDef TIMM_InitStructure;

	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_7,GPIO_P17,GROUP_AF_ODEFAULT);  //P17 used as TMIOA0 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_4,GPIO_P14,GROUP_AF_ODEFAULT);  //P14 used as TMIOB0 output(default function)

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_4 | GPIO_Pin_7 ;	//| GPIO_Pin_5 | GPIO_Pin_6
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;	
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;		
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);

	TIMM_InitStructure.TMM_Select 	= TMM0;	
	TIMM_InitStructure.TMM_Channel	= TMM_Channel_A|TMM_Channel_B|TMM_Channel_C|TMM_Channel_D;
	TIMM_InitStructure.TMM_Clk 		= clk;                       //specify the operation clk of tim
	TIMM_InitStructure.TMM_Mode		= TMM_Mode_Compare;                       // Compare mode
	
	TIMM_InitStructure.TMM_CHA_Pulse = period;  //Ϊperiod
	TIMM_InitStructure.TMM_OC.TMM_CHA.TMM_OCPolarity = TMM_Output_Toggle;   //TMﵽTMGRA,ƽзת

	TIMM_InitStructure.TMM_CHC_Pulse = n;		//TMﵽTMGRC(ֵΪn)ƽзת. ռձ:(period-n)/period
	TIMM_InitStructure.TMM_OC.TMM_CHC.TMM_OCPolarity = TMM_Output_Toggle;	 //
	TIMM_InitStructure.TMM_OC.TMM_CHC_OutPin_Ctrl = TMM_OC_Output_PinCtrl_Enable;//ͨȽĴTMGRCTMIOAi
	
	TIMM_InitStructure.TMM_CHB_Pulse = p;  
	TIMM_InitStructure.TMM_OC.TMM_CHB.TMM_OCPolarity = TMM_Output_Toggle;   //TMﵽTMGRB(ֵΪp),ƽзת

	TIMM_InitStructure.TMM_CHD_Pulse = q;		//TMﵽTMGRC(ֵΪq)ƽзת. ռձ:(p-q)/period
	TIMM_InitStructure.TMM_OC.TMM_CHD.TMM_OCPolarity = TMM_Output_Toggle;	 //
	TIMM_InitStructure.TMM_OC.TMM_CHD_OutPin_Ctrl = TMM_OC_Output_PinCtrl_Enable;//ͨȽĴTMGRDTMIOBi

	TIMM_InitStructure.TMM_CounterClear = TM_Clear_Match_TMGRA;  //when TM is matched with TMGRA, TM is cleared
	TIMM_InitStructure.TMM_Intp         = TMM_Int_IMIA; ///compare match interrupt enable A (IMIEA)
	
	TMM_Init(&TIMM_InitStructure);
	
	TMM0_Start(ENABLE);
}


/***********************************************************************************************************************
* Function Name: TMM0_PWM_1Perdod_3Duty
* @brief  The TMM0 operates as PWM output mode.
* @param  clk - select count source clock
*		@arg TMM_FCLK_Div1
*		@arg TMM_FCLK_Div2		
*		@arg TMM_FCLK_Div4		
*		@arg TMM_FCLK_Div8		
*		@arg TMM_FCLK_Div32		
*		@arg TMM_FCLK_TMCLK	
* @param  period - the period of PWM wave
* @param  duty1 - the duty of TMIOB0 output wave
* @param  duty2 - the duty of TMIOC0 output wave
* @param  duty3 - the duty of TMIOD0 output wave
* @return None
* @note   period = m, duty1 = n, duty2 = p, duty3 = q
*
*            ^
*       FFFFH|
*            |
*TMGRA0 = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
*            |                    _| |                    _| |
*            |                  _|   |                  _|   |
*            |                _|     |                _|     |
*TMGRB0 = n-1|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _
*            |            _| |       |            _| |       |
*TMGRC0 = p-1|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _
*            |        _| |   |       |        _| |   |       |        
*            |      _|   |   |       |      _|   |   |       |      _  
*TMGRD0 = q-1|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|    
*            |  _| |     |   |       |  _| |     |   |       |  _|      
*       0000H|_|___|_____|___|_______|_|___|_____|___|_______|_|______
*                  |     |   |       |     |     |   |       |
*            ______|_____|___|       |_____|_____|___|       |________
*     TMIOB0       |     |   |_______|     |     |   |_______|        
*            ______|_____|   |       |_____|_____|   |       |________
*     TMIOC0       |     |___|_______|     |     |___|_______|        
*            ______|     |   |       |_____|     |   |       |________
*     TMIOD0       |_____|___|_______|     |_____|___|_______|        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |__   |     |   |       |__
* TMSR0.IMFA ______|_____|___|_______|  |__|_____|___|_______|  |_____
*                  |     |   |__________   |     |   |__________
* TMSR0.IMFB ______|_____|___|          |__|_____|___|          |_____
*                  |     |______________   |     |______________
* TMSR0.IMFC ______|_____|              |__|_____|              |_____
*                  |____________________   |____________________
* TMSR0.IMFD ______|                    |__|                    |_____
*                                                 
***********************************************************************************************************************/
void TMM0_PWM_1Period_3Duty(uint8_t clk, uint16_t period, uint16_t duty1, uint16_t duty2, uint16_t duty3)
{
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMM_InitTypeDef TIMM_InitStructure;

	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_4,GPIO_P14,GROUP_AF_ODEFAULT);  //P14 used as TMIOB0 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_6,GPIO_P16,GROUP_AF_ODEFAULT);  //P16 used as TMIOC0 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_5,GPIO_P15,GROUP_AF_ODEFAULT);  //P15 used as TMIOD0 output(default function)
	
	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_4 | GPIO_Pin_6| GPIO_Pin_5 ;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;	
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;		
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);

	TIMM_InitStructure.TMM_Select 	= TMM0;	
	TIMM_InitStructure.TMM_Channel	= TMM_Channel_A|TMM_Channel_B|TMM_Channel_C|TMM_Channel_D;
	TIMM_InitStructure.TMM_Clk 		= clk;                       //specify the operation clk of tim
	TIMM_InitStructure.TMM_Mode		= TMM_Mode_PWM;                       // PWM mode
	
	TIMM_InitStructure.TMM_CHA_Pulse = period;  //TMGRAڣΪperiod

	TIMM_InitStructure.TMM_CHB_Pulse = duty1;  	//TMﵽTMGRB. ռձ:duty2/period
	TIMM_InitStructure.TMM_PWM.TMM_CHB.TMM_PWMInitLevel = TMM_PWMInitLevel_Low;
	TIMM_InitStructure.TMM_PWM.TMM_CHB.TMM_PWMActiveLevel = TMM_PWMActiveLevel_Low;
	
	TIMM_InitStructure.TMM_CHC_Pulse = duty2;	//TMﵽTMGRC. ռձ:duty2/period
	TIMM_InitStructure.TMM_PWM.TMM_CHC.TMM_PWMInitLevel = TMM_PWMInitLevel_Low;
	TIMM_InitStructure.TMM_PWM.TMM_CHB.TMM_PWMActiveLevel = TMM_PWMActiveLevel_Low;
	
	TIMM_InitStructure.TMM_CHD_Pulse = duty3;	//TMﵽTMGRD. ռձ:duty3/period
	TIMM_InitStructure.TMM_PWM.TMM_CHD.TMM_PWMInitLevel = TMM_PWMInitLevel_High;//TMM_PWMInitLevel_Low;//
	TIMM_InitStructure.TMM_PWM.TMM_CHD.TMM_PWMActiveLevel = TMM_PWMActiveLevel_High;//TMM_PWMActiveLevel_Low;//TMM_PWMActiveLevel_High;//
	
	TIMM_InitStructure.TMM_CounterClear = TM_Clear_Match_TMGRA;  //when TM is matched with TMGRA, TM0 is cleared
	TIMM_InitStructure.TMM_Intp         = TMM_Int_IMIA; ///compare match interrupt enable A (IMIEA)
	
	TMM_Init(&TIMM_InitStructure);
	
	TMM0_Start(ENABLE);
}

/***********************************************************************************************************************
* Function Name: TMM0_RstSynchronous_PWM
* @brief  output 6 pwm with same period .
* @param  clk - select count source clock
*		@arg TMM_FCLK_Div1
*		@arg TMM_FCLK_Div2		
*		@arg TMM_FCLK_Div4		
*		@arg TMM_FCLK_Div8		
*		@arg TMM_FCLK_Div32		
*		@arg TMM_FCLK_TMCLK	
* @param  period - the period of PWM wave
* @param  duty1 - the duty of TMIOB0 output wave
* @param  duty2 - the duty of TMIOA1 output wave
* @param  duty3 - the duty of TMIOB1 output wave
*            ^
*       FFFFH|
*            |
*TMGRA0 = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
*            |                    _| |                    _| |
*            |                  _|   |                  _|   |
*            |                _|     |                _|     |
*TMGRB0 = n-1|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _
*            |            _| |       |            _| |       |
*TMGRA1 = p-1|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _
*            |        _| |   |       |        _| |   |       |        
*            |      _|   |   |       |      _|   |   |       |      _  
*TMGRB1 = q-1|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|    
*            |  _| |     |   |       |  _| |     |   |       |  _|      
*       0000H|_|___|_____|___|_______|_|___|_____|___|_______|_|______
*                  |     |   |       |     |     |   |       |
*            ______|_____|___|       |_____|_____|___|       |________
*     TMIOB0       |     |   |_______|     |     |   |_______|        
*                  |     |   |       |     |     |   |       |
*                             _______|     |     |   |_______
*     TMIOD0 ______|_____|___|       |_____|_____|___|       |________        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |     |     |   |       |
*            ______|     |   |       |_____|     |   |       |________
*     TMIOB1       |_____|___|_______|     |_____|___|_______|        
*                  |     |   |       |     |     |   |       |
*                   _____|___|_______|     |_____|___|_______|
*     TMIOD1 ______|     |   |       |_____|     |   |       |________        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |     |     |   |       |
*            ______|_____|   |       |_____|_____|   |       |________
*     TMIOA1       |     |___|_______|     |     |___|_______|        
*                  |     |   |       |     |     |   |       |
*                        |___|_______|     |     |___|_______|
*     TMIOC1 ______|_____|   |       |_____|_____|   |       |_____        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |     |     |   |       |                                                
***********************************************************************************************************************/
void TMM0_RstSynchronous_PWM(uint8_t clk, uint16_t period, uint16_t duty1, uint16_t duty2, uint16_t duty3)
{
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMM_InitTypeDef TIMM_InitStructure;

	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_4,GPIO_P14,GROUP_AF_ODEFAULT);  //P14 used as TMIOB0 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_5,GPIO_P15,GROUP_AF_ODEFAULT);  //P15 used as TMIOD0 output(default function)
	
	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_4 | GPIO_Pin_5 ;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;	
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;		
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);

	TIMM_InitStructure.TMM_Select 	= TMM0;	
	TIMM_InitStructure.TMM_Channel	= TMM_Channel_A|TMM_Channel_B|TMM_Channel_D;
	TIMM_InitStructure.TMM_Clk 		= clk;                       //specify the operation clk of tim
	TIMM_InitStructure.TMM_Mode		= TMM_Mode_PWM_RSTSYN;        // RstSynchronous PWM mode
	
	TIMM_InitStructure.TMM_CHA_Pulse = period;  //TMGRAڣΪperiod

	TIMM_InitStructure.TMM_CHB_Pulse = duty1;  	//TMﵽTMGRB. ռձ:duty1/period
	TIMM_InitStructure.TMM_PWM.TMM_CHB.TMM_PWMInitLevel = TMM_PWMInitLevel_Low;
	TIMM_InitStructure.TMM_PWM.TMM_CHB.TMM_PWMActiveLevel = TMM_PWMActiveLevel_Low;
	
	TIMM_InitStructure.TMM_CHD_Pulse = duty1;	//TMﵽTMGRD. ռձ:duty1/period
	TIMM_InitStructure.TMM_PWM.TMM_CHD.TMM_PWMInitLevel = TMM_PWMInitLevel_Low;
	TIMM_InitStructure.TMM_PWM.TMM_CHD.TMM_PWMActiveLevel = TMM_PWMActiveLevel_Low;
	
	TIMM_InitStructure.TMM_CounterClear = TM_Clear_Match_TMGRA;  //when TM is matched with TMGRA, TM0 is cleared
	TIMM_InitStructure.TMM_Intp         = TMM_Int_IMIA; ///compare match interrupt enable A (IMIEA)
	
	TMM_Init(&TIMM_InitStructure);
	
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_0,GPIO_P10,GROUP_AF_ODEFAULT);  //P10 used as TMIOB1 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_1,GPIO_P11,GROUP_AF_ODEFAULT);  //P11 used as TMIOD1 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_2,GPIO_P12,GROUP_AF_ODEFAULT);  //P12 used as TMIOA1 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_3,GPIO_P13,GROUP_AF_ODEFAULT);  //P13 used as TMIOC1 output(default function)

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_0| GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_3;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;	
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);
	
	TIMM_InitStructure.TMM_Select 	= TMM1;	
	TIMM_InitStructure.TMM_Channel	= TMM_Channel_A|TMM_Channel_B|TMM_Channel_C|TMM_Channel_D;
	TIMM_InitStructure.TMM_Clk 		= clk;                       //specify the operation clk of tim
	TIMM_InitStructure.TMM_Mode		= TMM_Mode_PWM_RSTSYN;        // RstSynchronous PWM mode

	TIMM_InitStructure.TMM_CHA_Pulse = duty2;  //TMﵽTMGRA. ռձ:duty2/period
	TIMM_InitStructure.TMM_CHC_Pulse = duty2;  	//TMﵽTMGRB. ռձ:duty2/period

	TIMM_InitStructure.TMM_CHB_Pulse = duty3;  	//TMﵽTMGRB. ռձ:duty2/period
	TIMM_InitStructure.TMM_CHD_Pulse = duty3;  	//TMﵽTMGRB. ռձ:duty2/period
	
	TMM_Init(&TIMM_InitStructure);
	
	TMM0_Start(ENABLE);
}

/***********************************************************************************************************************
* Function Name: TMM0_PWM_Complementary_3Phase
* @brief  output 6 complementary pwm with dead time
* @param  clk - select count source clock
*		@arg TMM_FCLK_Div1
*		@arg TMM_FCLK_Div2		
*		@arg TMM_FCLK_Div4		
*		@arg TMM_FCLK_Div8		
*		@arg TMM_FCLK_Div32		
*		@arg TMM_FCLK_TMCLK	
* @param  half_period - the half period of PWM wave
* @param  duty1 - the half duty of TMIOB0 /TMIOD0 output wave
* @param  duty2 - the half duty of TMIOA1 /TMIOC1 output wave
* @param  duty3 - the half duty of TMIOB1 /TMIOD1 output wave
*       TMi  ^
*         m+1|_ _ _ _ _ _ _ _ _ _ _ _
*            |					  _| |_
*            |                  _|  _  |_ 
*            |                _|  _| |_  |_               
*            |              _|  _|     |_  |_            
*            |            _|  _|         |_  |_       
*     DUTY1  |_ _ _ _ _ _|_ _|_ _ _ _ _ _ _|_ _|_ 
*            |        _|  _| |             | |_  |_
*            |      _|  _|   |             |   |_  |_ 
*            |TM0 _|  _| |   |             |   | |_  |_         
*            |  _|  _|   |   |             |   |   |_  |_   _|
*        dead|_|_ _|_ _ _|_ _|_ _ _ _ _ _ _|_ _| __ _|_ _|_|     
*            |  _|       |   |             |   |       |_   
*       0000H|_|___TM1___|___|_______ _ ___|___|_____ ___|___________
*                        |   |             |   |            
*            ____________|___|             |___|_______       
*     TMIOB0             |   |_____________|   |       
*                        |                     |  
*                        |_____________________|
*     TMIOD0 ____________|                     |____________   
*                                                              
***********************************************************************************************************************/
void TMM0_PWM_Complementary_3Phase(uint8_t clk, uint16_t half_period,uint16_t duty1,uint16_t duty2,uint16_t duty3, uint16_t dead)
{
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMM_InitTypeDef TIMM_InitStructure;

	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_5,GPIO_P15,GROUP_AF_ODEFAULT);  //P15 used as TMIOD0 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_4,GPIO_P14,GROUP_AF_ODEFAULT);  //P14 used as TMIOB0 output(default function)
	
	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_4 | GPIO_Pin_5 ;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;	
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;		
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);

	TIMM_InitStructure.TMM_Select 	= TMM0;	
	TIMM_InitStructure.TMM_Channel	= TMM_Channel_A|TMM_Channel_B|TMM_Channel_D;
	TIMM_InitStructure.TMM_Clk 		= clk;                       //specify the operation clk of tim
	TIMM_InitStructure.TMM_Mode		= TMM_Mode_CompPWM;        //  complementary PWM mode

	TIMM_InitStructure.TMM_BDTR.TMM_PostiveState = TMM_InitHigh_ActiveLow; //PWM ƽãʼΪߣЧΪ͵ƽ
	TIMM_InitStructure.TMM_BDTR.TMM_NegativeState = TMM_InitHigh_ActiveLow;//PWM ƽãʼΪߣЧΪ͵ƽ
	TIMM_InitStructure.TMM_BDTR.TMM_DeadTime = dead; 						//ʱ
	TIMM_InitStructure.TMM_Combine_TransMode = TMM_Trans_TM1_Underflow; // TM1ʱ򣬻ĴݰƵͨüĴ
	
	TIMM_InitStructure.TMM_CHA_Pulse = half_period ;  //TMGRAڣڻPWMΪĶģʽΪperiod = half_period *2
	TIMM_InitStructure.TMM_CHB_Pulse = duty1;         
	TIMM_InitStructure.TMM_CHD_Pulse = duty1;
	TIMM_InitStructure.TMM_CounterClear = TM_Clear_Disable;  // TM0  counter is free running	
	TIMM_InitStructure.TMM_Intp         = TMM_Int_IMIA; //compare match interrupt enable A (IMIEA) when TM0 match with TMGRA0
	TMM_Init(&TIMM_InitStructure);

	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_0,GPIO_P10,GROUP_AF_ODEFAULT);  //P10 used as TMIOB1 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_1,GPIO_P11,GROUP_AF_ODEFAULT);  //P11 used as TMIOD1 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_2,GPIO_P12,GROUP_AF_ODEFAULT);  //P12 used as TMIOA1 output(default function)
	GPIO_PinAFConfig(GPIO_PORT1,GPIO_Pin_3,GPIO_P13,GROUP_AF_ODEFAULT);  //P13 used as TMIOC1 output(default function)

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_0| GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_3;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;	
	GPIO_Init(GPIO_PORT1,&GPIO_InitStruct);
	
	TIMM_InitStructure.TMM_Select 	= TMM1;	
	TIMM_InitStructure.TMM_Channel	= TMM_Channel_A|TMM_Channel_B|TMM_Channel_C|TMM_Channel_D;

	TIMM_InitStructure.TMM_BDTR.TMM_PostiveState = TMM_InitHigh_ActiveLow; //PWM ƽãʼΪߣЧΪ͵ƽ
	TIMM_InitStructure.TMM_BDTR.TMM_NegativeState = TMM_InitHigh_ActiveLow;//PWM ƽãʼΪߣЧΪ͵ƽ
	TIMM_InitStructure.TMM_Combine_TransMode = TMM_Trans_TM1_Underflow; // TM1ʱ򣬻ĴݰƵͨüĴ

	TIMM_InitStructure.TMM_CHA_Pulse = duty2;  //TMﵽTMGRA. ռձ:duty2/period
	TIMM_InitStructure.TMM_CHC_Pulse = duty2;  	//TMﵽTMGRB. ռձ:duty2/period

	TIMM_InitStructure.TMM_CHB_Pulse = duty3;  	//TMﵽTMGRB. ռձ:duty2/period
	TIMM_InitStructure.TMM_CHD_Pulse = duty3;  	//TMﵽTMGRB. ռձ:duty2/period
	TMM_Init(&TIMM_InitStructure);

	ISR_Register(TMM0_IRQn, tmm0_interrupt);  //жϷ·ע
	
	TMM_All_Start();
}



/***********************************************************************************************************************
* Function Name: tmm0_interrupt
* @brief  timm0 interrupt service routine for input capture mode
* @param  None
* @return None
***********************************************************************************************************************/
void tmm0_interrupt(void)
{
    INTC_ClearPendingIRQ(TMM0_IRQn); /* clear INTTMM0 interrupt flag */
    uint8_t  tmsr0_temp = TMM->TMSR0;
    uint8_t  tmier0_temp = TMM->TMIER0;

    TMM->TMIER0 = 0x00U;

    if (TMM_GetFlagStatus(TMM0,TMM_FLAG_OVFLOW) == SET)
    {
        TMM->TMSR0 = tmsr0_temp & (uint8_t)~TMM_FLAG_OVFLOW;
    }

    if (TMM_GetFlagStatus(TMM0,TMM_FLAG_MATCHA) == SET)
    {
        TMM->TMSR0 = tmsr0_temp & (uint8_t)~TMM_FLAG_MATCHA;
        g_tmm0_intTaken_a++;
		interval_time_flag = 1;
    }
    if (TMM_GetFlagStatus(TMM0,TMM_FLAG_MATCHB) == SET)
    {
        TMM->TMSR0 = tmsr0_temp & (uint8_t)~(TMM_FLAG_MATCHB);
    }
    if (TMM_GetFlagStatus(TMM0,TMM_FLAG_MATCHC) == SET)
    {
        TMM->TMSR0 = tmsr0_temp & (uint8_t)~TMM_FLAG_MATCHC;		
    }
    if (TMM_GetFlagStatus(TMM0,TMM_FLAG_MATCHD) == SET)
    {
        TMM->TMSR0 = tmsr0_temp & (uint8_t)~TMM_FLAG_MATCHD;
    }

    TMM->TMIER0 = tmier0_temp;
}
